Texture2D txDiffuse : register( t0 );
SamplerState samLinear : register( s0 );

cbuffer cbNeverChanges : register( b0 )
{
    matrix View;
};

cbuffer cbChangeOnResize : register( b1 )
{
    matrix Projection;
};

cbuffer cbChangesEveryFrame : register( b2 )
{
    matrix World;
};
cbuffer CBSpriteData : register( b3 )
{
	float4 posx;
	float4 posy;
	float4 posz;
	float4 texu;
	float4 texv;
	int posmode;
	float3 dummy;
};
cbuffer CBPointData : register( b4 )
{
	float4 pos;
	float4 pointcolor;
};
cbuffer CBLineData : register( b5 )
{
	float4 pos0;
	float4 pointcolor0;
	float4 pos1;
	float4 pointcolor1;
};
cbuffer CBParticleData : register( b6 )
{
	float4 posp;
	float widthp;
	float heightp;
	float2 texup;
	float2 texvp;
	float2 dummyp;
};
//--------------------------------------------------------------------------------------
struct VS_INPUT
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD0;
};

struct PS_INPUT
{
    float4 Pos : SV_POSITION;
    float2 Tex : TEXCOORD0;
};
struct PS_INPUTpoint
{
    float4 Pos : SV_POSITION;
    float4 Col : COLOR;
};

//--------------------------------------------------------------------------------------
// Vertex Shader///--------------------------------------------------------------------------------------
PS_INPUT VS( VS_INPUT input )
{
    PS_INPUT output = (PS_INPUT)0;
    output.Pos = mul( input.Pos, World );
    output.Pos = mul( output.Pos, View );
    output.Pos = mul( output.Pos, Projection );
    output.Tex = input.Tex;
    
    return output;
}
PS_INPUT VSsprite(uint vID : SV_VertexID)
{
	float4 pos = 0;
	float2 tex = 0;
	PS_INPUT output = (PS_INPUT)0;
	switch (vID)
	{
    case 0: pos = float4(posx.x,posy.x,posz.x,1.0); tex = float2(texu.x,texv.x); break;
    case 1: pos = float4(posx.y,posy.y,posz.y,1.0); tex = float2(texu.y,texv.y); break;
    case 2: pos = float4(posx.z,posy.z,posz.z,1.0); tex = float2(texu.z,texv.z); break;
    case 3: pos = float4(posx.w,posy.w,posz.w,1.0); tex = float2(texu.w,texv.w); break;
    }
    if (posmode == 0){
	    output.Pos = pos;
	}
	else{
		output.Pos = mul( pos, World );
		output.Pos = mul( output.Pos, View );
		output.Pos = mul( output.Pos, Projection );
	}
    output.Tex = tex;
    return output;
}
PS_INPUTpoint VSpoint(uint vID : SV_VertexID)
{
	PS_INPUTpoint output = (PS_INPUTpoint)0;
	output.Pos = pos;
    output.Col = pointcolor;
    return output;
	
}
PS_INPUTpoint VSline(uint vID : SV_VertexID)
{
	PS_INPUTpoint output = (PS_INPUTpoint)0;
	switch (vID)
	{
    case 0: output.Pos = pos0; output.Col = pointcolor0; break;
    case 1: output.Pos = pos1; output.Col = pointcolor1; break;
    }
    return output;
	
}
PS_INPUT VSparticle(uint vID : SV_VertexID)
{
	float2 tex = 0;
	PS_INPUT output = (PS_INPUT)0;
	
	output.Pos = mul( posp, World );
	output.Pos = mul( output.Pos, View );
    output.Tex = tex;
    return output;
}
[maxvertexcount(4)]
void GS_PARTICLE(point PS_INPUT In[1],
                inout TriangleStream<PS_INPUT> ParticleStream)
{
    // _ʂɂ
    float4 posLT = In[0].Pos + float4(-widthp * 0.5,heightp * 0.5, 0.0, 0.0);
    float4 posLB = In[0].Pos + float4(-widthp * 0.5,-heightp * 0.5, 0.0, 0.0);
    float4 posRT = In[0].Pos + float4(widthp * 0.5,heightp * 0.5, 0.0, 0.0);
    float4 posRB = In[0].Pos + float4(widthp * 0.5,-heightp * 0.5, 0.0, 0.0);

    // ̓_̈ʒu(ˉeWn)vZďo
    PS_INPUT Out;
    Out.Pos = mul(posLT, Projection);
    Out.Tex = float2(texup.x,texvp.x);
    ParticleStream.Append(Out);

    // E̓_̈ʒu(ˉeWn)vZďo
    Out.Pos = mul(posRT, Projection);
    Out.Tex = float2(texup.y,texvp.x);
    ParticleStream.Append(Out);

    // ̓_̈ʒu(ˉeWn)vZďo
    PS_INPUT Out1;
    Out1.Pos = mul(posLB, Projection);
    Out1.Tex = float2(texup.x,texvp.y);
    ParticleStream.Append(Out1);

    // E̓_̈ʒu(ˉeWn)vZďo
    Out.Pos = mul(posRB, Projection);
    Out.Tex = float2(texup.y,texvp.y);
    ParticleStream.Append(Out);

    ParticleStream.RestartStrip();
}

//--------------------------------------------------------------------------------------
// Pixel Shader
//--------------------------------------------------------------------------------------
float4 PS( PS_INPUT input) : SV_Target
{
	float4 outputcol = 0;
	outputcol = txDiffuse.Sample( samLinear, input.Tex );
	if (outputcol.x == 0.0 && outputcol.y == 0.0 &&outputcol.z == 0.0) discard;
    return outputcol;
}
float4 PSpoint( PS_INPUTpoint input) : SV_Target
{
    return input.Col;
}
float4 PSddsalpha( PS_INPUT input) : SV_Target
{
	float4 outputcol = 0;
	outputcol = txDiffuse.Sample( samLinear, input.Tex );
	if (outputcol.w < 1.0) discard;
    return outputcol;
}
